<?php

namespace App\Http\Controllers;

use App\Http\Requests;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Session;
use App\Models\User;

class AdminController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */

    public $user;

    public function __construct()
    {
        $this->middleware('auth');

        $this->user = Auth::user();
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Http\Response
     */
    public function addAdmin()
    {
        return view('admin.add');
    }

    public function saveAdmin(Request $request)
    {
        $name = $request->name;
        $email = $request->email;
        $password = $request->password;
        $confirmPassword = $request->confirmPassword;

        if($password!=$confirmPassword){
            Session::flash('alert-danger', 'Confirm Password Did Not Match');
            return redirect()->back()->withInput();
        }

        $newUser = new User();
        $newUser->name = $name;
        $newUser->email = $email;
        $newUser->password = bcrypt($password);
        $newUser->save();

        Session::flash('alert-success', 'User created successfully');
        return redirect('admin-list');
    }

    public function adminList()
    {
        if(!empty($this->user) && $this->user->user_type == 'administrator'){
            $adminList = User::all();
        }else{
            $adminList = User::where('user_type', '!=', 'administrator')->get();
        }


        $data['adminList'] = $adminList;

        return view('admin.list',$data);
    }

    public function editAdmin($id)
    {
        if(!empty($this->user) && $this->user->user_type == 'administrator'){
            $user = User::where('id',$id)->first();
        }else{
            $user = User::where('id',$id)->where('user_type', '!=', 'administrator')->first();
            if(empty($user)){
                return redirect('/')->with('alert-danger','You have not administrator access');
            }
        }

        $data["user"] = $user;
        return view('admin.edit',$data);
    }

    public function updateAdmin(Request $request, $id)
    {
        $name = $request->name;
        $email = $request->email;
        $password = $request->password;
        $confirmPassword = $request->confirmPassword;

        if($password!='' || $confirmPassword!='' ){
            if($password!=$confirmPassword){
                Session::flash('alert-danger', 'Confirm Password Did Not Match');
                return redirect()->back()->withInput();
            }
        }
        

        $updateUser = User::where('id',$id)->first();
        $updateUser->name = $name;
        $updateUser->email = $email;
        if($password!=''){
            $updateUser->password = bcrypt($password);
        }
        
        $updateUser->save();

        Session::flash('alert-success', 'User created successfully');

        return redirect('admin-list');
    }

    public function deleteAdmin($id)
    {
        $user = User::where('id',$id)->first();
        $user->delete();
        return redirect('admin-list');
    }
}
