<?php

namespace App\Http\Controllers;

use App\Http\Requests;
use App\Model\IntCountryCodes;
use App\Model\Customer;
use App\Model\CustomerGroup;
use App\Model\CustomerGroupCustomer;
use App\Model\GroupBroadcast;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Input;
use Illuminate\Support\Facades\Response;
use Maatwebsite\Excel\Facades\Excel;
use Illuminate\Support\Facades\Session;
use Exception;
use function GuzzleHttp\Psr7\str;
use Maatwebsite\Excel\Concerns\WithDrawings;

class CustomerController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth');
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Http\Response
     */
    public function addCustomer()
    {
        $country_code = IntCountryCodes::where('country_code', '880')->select('country_code', 'country_name')->get();
        $customerGroup = CustomerGroup::orderBy('created_at','Desc')->get();
        $broadCastGroup = GroupBroadcast::orderBy('created_at','Desc')->get();
     
        $data["customerGroup"] = $customerGroup;
        $data["broadCastGroup"] = $broadCastGroup;
        $data["country_code"] = $country_code;

        return view('customers.add-customer', $data);
    }

    public function storeNewCustomer(Request $request)
    {
        $this->validate($request, [
            "country_code" => 'required',
            "customer_group" => 'required',
            "broadcast_group" => 'required',
            "number" => 'required',

        ]);

        $checkCustomer = Customer::where('number',request('number'))->first();

        if($checkCustomer==""){
            $customer = new Customer;
            //$customer->phone = "+".request('country_code').request('number');
            $customer->code = request('country_code');
            $customer->number = request('number');
            $customer->first_name = request('first_name');
            $customer->last_name = request('last_name');
            $customer->email = request('email');
            $customer->gender = request('gender');
            $customer->wp = request('wp');
            $customer->user_name = request('username');
            $customer->status = 0;
            $customer->opt_out_db = "no";
            $customer->save();
            $customerId = $customer->id;
        }else{
            $customerId = $checkCustomer->id;
        }
        
        $customerGroups = $request->customer_group;
        $broadcastGroup = $request->broadcast_group;
        
        if(count($customerGroups)>0){

            foreach ($customerGroups as $key => $value) {

                $checkCustomerGroupCustomer = CustomerGroupCustomer::where('customer_group_id',$value)->where('customer_id',$customerId)->where('group_broadcast_id',$broadcastGroup)->first();

                if($checkCustomerGroupCustomer==""){
                    $customer = new CustomerGroupCustomer;
                    $customer->customer_group_id = $value;
                    $customer->customer_id = $customerId;
                    $customer->group_broadcast_id = $broadcastGroup;
                    $customer->save();
                }
                
            }
        }
        Session::flash('alert-success', 'Customer added successfully');
        return redirect('/customers');
    }

    public function getCustomers(Request $request)
    {
        $limit = 10;
        $search_code = '';
        $search_phone_number = '';
        $search_user_name = '';
        $search_whatsapp_availability = '';
        $search_customer_group = [];
        $search_customer_broadcast = [];
      
        if($request->phone_number || $request->user_name || $request->customer_group || $request->customer_broadcast || $request->code || $request->whatsapp_availability!=''){
         
            $search_phone_number = $request->phone_number;
            $search_user_name = $request->user_name;
            $search_code = $request->code;
            $search_whatsapp_availability = $request->whatsapp_availability;
            
            if($request->customer_group && count($request->customer_group)>0){
                $search_customer_group = $request->customer_group;
            }

            if($request->customer_broadcast && count($request->customer_broadcast)>0){
                $search_customer_broadcast = $request->customer_broadcast;
            }
         
            
            $customersQuery = Customer::query();
            
            if ($search_code != '') {
                $customersQuery->where('code', 'like', '%' . $search_code . '%');
            }
            if ($request->phone_number != '') {
                $customersQuery->where('number', 'like', '%' . $request->phone_number . '%');
            }
            if($request->user_name){
                $customersQuery->where('user_name', 'like', '%' . $request->user_name . '%');
            }

            
            if($request->whatsapp_availability!=''){
               
                $customersQuery->where('wp', $request->whatsapp_availability);
            }

            if($search_customer_broadcast){
                $broadcastCustomerId = CustomerGroupCustomer::whereIn('group_broadcast_id', $search_customer_broadcast)->pluck('customer_id')->toArray();
                $customersQuery->whereIn('id',$broadcastCustomerId);
            }

            if($search_customer_group){
                $customerId = CustomerGroupCustomer::whereIn('customer_group_id', $search_customer_group)->pluck('customer_id')->toArray();
 
                $customersQuery->whereIn('id',$customerId);
            }

            
            
   
            $customers = $customersQuery->where('status', 0)->where('opt_out_db', 'no')->orderBy('created_at','Desc')->paginate($limit);
           
        }else{
            $customers = Customer::where('status', 0)->where('opt_out_db', 'no')->orderBy('created_at','Desc')->paginate($limit);
        }


 
        $groups = CustomerGroup::orderBy('created_at', 'desc')->get();
        $broadcast = GroupBroadcast::orderBy('created_at', 'desc')->get();

        $data['customerGroup'] = $groups;
        $data['customerBroadcast'] = $broadcast;
        $data["customers"] = $customers;
        $data["page"] = $customers->currentPage();
        $data["limit"] = $limit;
        $data["search_phone_number"] = $search_phone_number;
        $data["search_user_name"] = $search_user_name;
        $data["search_customer_group"] = $search_customer_group;
        $data["search_customer_broadcast"] = $search_customer_broadcast;
        $data["search_code"] = $search_code;
        $data["search_whatsapp_availability"] = $search_whatsapp_availability;

        $data["search_customer_group_string"] = $search_customer_group && count($search_customer_group)>0 ? implode(",",$search_customer_group):'';
        $data["search_customer_broadcast_string"] = $search_customer_broadcast && count($search_customer_broadcast)>0 ? implode(",",$search_customer_broadcast):'';
 
        return view('customers.customers', $data);
    }

    public function getCustomer($id)
    {
        $customer = Customer::find($id);
        $country_code = IntCountryCodes::where('Active', '1')->select('country_code', 'country_name')->get();
        $customerGroup = CustomerGroup::all();
        $customerGroupCustomer = CustomerGroupCustomer::where('customer_id',$id)->pluck('customer_group_id')->toArray();
        $customerGroupBroadcastId = CustomerGroupCustomer::where('customer_id',$id)->pluck('group_broadcast_id')->toArray();
        $broadCastGroup = GroupBroadcast::orderBy('created_at','Desc')->get();

        $data["customerGroupCustomer"] = $customerGroupCustomer;
        $data["customerGroupBroadcastId"] = $customerGroupBroadcastId[0];
        $data["customerGroup"] = $customerGroup;
        $data["broadCastGroup"] = $broadCastGroup;
        $data["customer"] = $customer;
        $data["country_code"] = $country_code;
        return view('customers.customer', $data);
    }

    public function updateCustomer(Request $request, $id)
    {
        $this->validate($request, [
            "country_code" => 'required',
            "customer_group" => 'required',
            "broadcast_group" => 'required',
            "number" => 'required|numeric|unique:customers,number,' . $id,
        ]);

        $customerGroups = $request->customer_group;
        $broadcastGroup = $request->broadcast_group;

        $customer = Customer::find($id);

        $customer->code = request('country_code');
        $customer->number = request('number');
        $customer->first_name = request('first_name');
        $customer->last_name = request('last_name');
        $customer->email = request('email');
        $customer->gender = request('gender');
        $customer->user_name = request('username');
        $customer->wp = request('wp');
        $customer->save();
        $customerId = $customer->id;

        $groups = CustomerGroupCustomer::where('customer_id',$customerId)->delete();

        

        if(count($customerGroups)>0){

            foreach ($customerGroups as $key => $value) {
                $customer = new CustomerGroupCustomer;
                $customer->customer_group_id = $value;
                $customer->customer_id = $customerId;
                $customer->group_broadcast_id = $request->broadcast_group;
                $customer->save();
            }
        }
        Session::flash('alert-success', 'Customer updated successfully');
        return redirect('/customers');

    }

    public function deleteCustomer($id)
    {
        $customer = Customer::find($id);
        $customer->delete();
        Session::flash('alert-success', 'Customer deleted successfully');
        return redirect('/customers');
    }


    public function customersGroups()
    {
        $limit = 10;
        $groups = CustomerGroup::with('broadcast')->withCount('customers')->orderBy('created_at', 'desc')->paginate($limit);
        // return $groups;
        $data['groups'] = $groups;
        $data["page"] = $groups->currentPage();
        $data["limit"] = $limit;
        return view('customers.customers-groups', $data);
    }

    public function createGroup(Request $request)
    {
        $this->validate($request, [
            "group_name" => 'required'
        ]);

        $group = new CustomerGroup;
        $group->name = request('group_name');
        $group->save();
        Session::flash('alert-success', 'Group successfully created.');
        return redirect('/customers-groups');
    }
    
    
    public function createGroupAddCustomer(Request $request)
    {
        $this->validate($request, [
            "group_name" => 'required'
        ]);

        $group = new CustomerGroup;
        $group->name = request('group_name');
        $group->save();
        Session::flash('alert-success', 'Group successfully created.');
        return redirect('/add-customer');
    }
    
    public function createBroadcastAddCustomer(Request $request)
    {
        $this->validate($request, [
            "broadcast_name" => 'required',
            "broadcast_id" => 'required',
            "customer_group" => 'required',
        ]);

        $checkBroadcast = GroupBroadcast::where('broadcast_id',request('broadcast_id'))->first();

        if($checkBroadcast!=''){
            Session::flash('alert-warning', 'Broadcast Id already exist.');
            return redirect('/add-customer');
        }

        $group = new GroupBroadcast;
        $group->broadcast_name = request('broadcast_name');
        $group->broadcast_id = request('broadcast_id');
        $group->group_id = request('customer_group');
        $group->save();
        Session::flash('alert-success', 'Broadcast successfully created.');
        return redirect('/add-customer');
    }

    public function updateGroupName(Request $request, $id)
    {
        $this->validate($request, [
            "group_name" => 'required',
        ]);

        $group = CustomerGroup::find($id);

        $group->name = request('group_name');
        $group->save();
        Session::flash('alert-success', 'Group name successfully updated.');
        return redirect('/customers-groups');

    }

    public function deleteGroup($id)
    {
        CustomerGroupCustomer::where('customer_group_id', $id)->delete();
        $group = CustomerGroup::find($id);
        $group->delete();
        Session::flash('alert-success', 'Group successfully deleted.');
        return redirect('/customers-groups');
    }

    public function addCustomerToGroup($id)
    {
        $limit=10;
        $existingCustomerIds = CustomerGroupCustomer::where('customer_group_id', $id)->pluck('customer_id')->toArray();
        $customerIds = DB::table('customers')->whereNotIn('id', $existingCustomerIds)->paginate($limit);

        $data["customers"] = $customerIds;
        $data["groupId"] = $id;
        $data["page"] = $customerIds->currentPage();
        $data["limit"] = $limit;

        return view('customers.add-customer-to-group', $data);
    }

    public function saveCustomerToGroup($groupId, $customerId)
    {

        $customer = new CustomerGroupCustomer;
        $customer->customer_group_id = $groupId;
        $customer->customer_id = $customerId;
        $customer->save();
        Session::flash('alert-success', 'Customer added successfully to group.');
        return redirect()->back();
    }

    public function saveAllCustomerToGroup($groupId)
    {
        $customerIds = Input::get('idList');

        for ($i = 0; $i < sizeof($customerIds); $i++) {
            $customer = new CustomerGroupCustomer;
            $customer->customer_group_id = $groupId;
            $customer->customer_id = $customerIds[$i];
            $customer->save();
        }
    }

    public function viewCustomers($id)
    {
        $limit = 10;
        $group = CustomerGroup::find($id);
        $customers = CustomerGroupCustomer::where('customer_group_id', $id)->get();
        $customerIds = [];
        
        // CustomerGroup::with('broadcast')->withCount('customers')->orderBy('created_at', 'desc')->paginate($limit);

        foreach ($customers as $customer) {
            array_push($customerIds, $customer->customer_id);
        }

        $inGroupCustomers = DB::table('customers')->whereIn('id', $customerIds)->orderBy('created_at',"Desc")->paginate($limit);

        $data["customers"] = $inGroupCustomers;
        $data["groupId"] = $id;
        $data["group"] = $group;
        $data["page"] = $inGroupCustomers->currentPage();
        $data["limit"] = $limit;
        
        return view('customers.view-group-customers', $data);
    }

    public function deleteCustomerFromGroup($groupId, $customerId)
    {
        $customer = CustomerGroupCustomer::where('customer_group_id', $groupId)
            ->where('customer_id', $customerId)->delete();
        Session::flash('alert-success', 'Customer successfully deleted from group.');
        return redirect()->back();
    }

    public function deleteAllCustomerFromGroup($groupId)
    {
        $customerIds = Input::get('idList');
        $customer = CustomerGroupCustomer::where('customer_group_id', $groupId)
            ->whereIn('customer_id', $customerIds)->delete();
    }

    public function getExportCustomers()
    {
        $customers = Customer::orderBy('id', 'desc')->get();
        $data["customers"] = $customers;
        return view('customers.export-customers', $data);
    }



    public function listedCustomer($broadcastId)
    {
        $limit = 10;
        $customerIds = CustomerGroupCustomer::where('group_broadcast_id', $broadcastId)->pluck('customer_id')->toArray();
        $inGroupCustomers = DB::table('customers')->whereIn('id', $customerIds)->where('wp',1)->orderBy('created_at',"Desc")->paginate($limit);

        $data["customers"] = $inGroupCustomers;
        $data["groupId"] = $broadcastId;
        $data["page"] = $inGroupCustomers->currentPage();
        $data["limit"] = $limit;
        return view('customers.listed-customers', $data);
    }


    // public function exportCustomersCsvFile()
    // {
    //     $customerIds = Input::get('idList');
      
    //     $table = DB::table('customers')->whereIn('id', $customerIds)->get();
    //    $result = $table;
    //     $baseUrl = public_path();
    //     $filename = $baseUrl . '/assets/exported-customers.xls';
    //     $handle = fopen($filename, 'w+');
    //     fputcsv($handle, array('First Name', 'Last Name', 'Email', 'Club Account', 'Phone Number'));

    //     foreach ($result as $row) {
    //         fputcsv($handle, array($row->first_name, $row->last_name, $row->email, $row->user_name, $row->code . $row->number));
    //     }

    //     fclose($handle);

    //     $headers = array(
    //         'Content-Type' => 'text/csv',
    //     );


    // }


    public function exportAllCustomersCsvFile(Request $request)
    {

        $search_code = '';
        $search_phone_number = '';
        $search_user_name = '';
        $search_customer_group = [];
        $search_customer_broadcast = [];
        
        if($request->phone_number || $request->user_name || $request->customer_group || $request->customer_broadcast || $request->code || $request->whatsapp_availability!=''){
            
            $search_phone_number = $request->phone_number;
            $search_user_name = $request->user_name;
            $search_code = $request->code;
            
            if($request->customer_group){
                $search_customer_group = explode (",", $request->customer_group);
            }

            if($request->customer_broadcast){
                $search_customer_broadcast = explode (",", $request->customer_broadcast);
            }
            
            
            $customersQuery = Customer::query();
            
            if ($search_code != '') {
                $customersQuery->where('code', 'like', '%' . $search_code . '%');
            }
            if ($request->phone_number != '') {
                $customersQuery->where('number', 'like', '%' . $request->phone_number . '%');
            }
            if($request->user_name){
                $customersQuery->where('user_name', 'like', '%' . $request->user_name . '%');
            }

            if($search_customer_group){
                $customerId = CustomerGroupCustomer::whereIn('customer_group_id', $search_customer_group)->pluck('customer_id')->toArray();
               
                $customersQuery->whereIn('id',$customerId);
            }

            if($request->whatsapp_availability!=''){
               
                $customersQuery->where('wp', $request->whatsapp_availability);
            }

            if($search_customer_broadcast){
                $broadcastCustomerId = CustomerGroupCustomer::whereIn('group_broadcast_id', $search_customer_broadcast)->pluck('customer_id')->toArray();
                $customersQuery->whereIn('id',$broadcastCustomerId);
            }


            $customers = $customersQuery->where('status', 0)->where('opt_out_db', 'no')->orderBy('created_at','Desc')->get();
           
        }else{
            $customers = Customer::where('status', 0)->where('opt_out_db', 'no')->orderBy('created_at','Desc')->get();
        }
   
        $output= Excel::create('customer-list', function($excel) use ($customers) {
            $excel->sheet('sheet1', function($sheet) use ($customers)
            {
                $sheet->setColumnFormat(array(
                    'E' => '0',
                ));

                $sheet->mergeCells('A1:E2');
                $sheet->cell('A1', function($cell) {
                    $cell->setValue('GULSHAN CLUB LIMITED');  
                    $cell->setBackground('#33CCCC');  
                    $cell->setAlignment('center');
                    $cell->setFontWeight('bold');    
                    $cell->setFontSize('20');    
                });

                $sheet->mergeCells('A3:E4');
                $sheet->cell('A3', function($cell) {
                    $cell->setValue('Customer List');  
                    $cell->setBackground('#008686');  
                    $cell->setAlignment('center');
                    $cell->setFontWeight('bold');    
                    $cell->setFontSize('20');    
                });

                $sheet->cell('A5', function($cell) {$cell->setValue('No.');  $cell->setFontWeight('bold');});
                $sheet->cell('B5', function($cell) {$cell->setValue('Name');  $cell->setFontWeight('bold');});
                // $sheet->cell('C4', function($cell) {$cell->setValue('Last Name'); $cell->setFontWeight('bold');  });
                $sheet->cell('C5', function($cell) {$cell->setValue('Email'); $cell->setFontWeight('bold');  });
                $sheet->cell('D5', function($cell) {$cell->setValue('Club Account'); $cell->setFontWeight('bold');  });
                $sheet->cell('E5', function($cell) {$cell->setValue('Phone Number'); $cell->setFontWeight('bold');  });

                $i=6;

                foreach ($customers as $key => $value) {
                    $sheet->cell('A'.$i, $key+1); 
                    $sheet->cell('B'.$i, $value['first_name']); 
                    // $sheet->cell('C'.$i, $value['last_name']); 
                    $sheet->cell('C'.$i, $value['email']); 
                    $sheet->cell('D'.$i, $value['user_name']); 
                    $sheet->cell('E'.$i, $value['code'].$value['number']); 
                    $i++;
                }

                $i++;
                $sheet->mergeCells('A'.$i.':E'.$i);
                $sheet->cell('A'.$i, function($cell) {
                    $cell->setValue('This file is software generated');  
                    $cell->setBackground('#6666CC');  
                    $cell->setAlignment('center');
                });
                $i++;
                $sheet->mergeCells('A'.$i.':E'.$i);
                $sheet->cell('A'.$i, function($cell) {
                    $cell->setValue('Powered By WORKSPACE INFOTECH LIMITED');  
                    $cell->setBackground('#33CCCC');  
                    $cell->setAlignment('center');
                });
            });
       })->download('xlsx');

    }

    public function exportGroupCustomersCsvFile($id)
    {
        
        $customerIds = CustomerGroupCustomer::where('customer_group_id',$id)->pluck('customer_id')->toArray();
      
        $customers = DB::table('customers')->whereIn('id', $customerIds)->get();
       
        $output= Excel::create('Group-customer-list', function($excel) use ($customers) {
            $excel->sheet('sheet1', function($sheet) use ($customers)
            {
                $sheet->setColumnFormat(array(
                    'E' => '0',
                ));

                $sheet->mergeCells('A1:E2');
                $sheet->cell('A1', function($cell) {
                    $cell->setValue('GULSHAN CLUB LIMITED');  
                    $cell->setBackground('#33CCCC');  
                    $cell->setAlignment('center');
                    $cell->setFontWeight('bold');    
                    $cell->setFontSize('20');    
                });

                $sheet->mergeCells('A3:E4');
                $sheet->cell('A3', function($cell) {
                    $cell->setValue('Group Customer List');  
                    $cell->setBackground('#008686');  
                    $cell->setAlignment('center');
                    $cell->setFontWeight('bold');    
                    $cell->setFontSize('20');    
                });

                $sheet->cell('A5', function($cell) {$cell->setValue('No.');  $cell->setFontWeight('bold');});
                $sheet->cell('B5', function($cell) {$cell->setValue('Name');  $cell->setFontWeight('bold');});
                // $sheet->cell('C4', function($cell) {$cell->setValue('Last Name'); $cell->setFontWeight('bold');  });
                $sheet->cell('C5', function($cell) {$cell->setValue('Email'); $cell->setFontWeight('bold');  });
                $sheet->cell('D5', function($cell) {$cell->setValue('Club Account'); $cell->setFontWeight('bold');  });
                $sheet->cell('E5', function($cell) {$cell->setValue('Phone Number'); $cell->setFontWeight('bold');  });

                $i=6;

                foreach ($customers as $key => $value) {
                    $sheet->cell('A'.$i, $key+1); 
                    $sheet->cell('B'.$i, $value->first_name); 
                    // $sheet->cell('C'.$i, $value['last_name']); 
                    $sheet->cell('C'.$i, $value->email); 
                    $sheet->cell('D'.$i, $value->user_name); 
                    $sheet->cell('E'.$i, $value->code.$value->number); 
                    $i++;
                }

                $i++;
                $sheet->mergeCells('A'.$i.':E'.$i);
                $sheet->cell('A'.$i, function($cell) {
                    $cell->setValue('This file is software generated');  
                    $cell->setBackground('#6666CC');  
                    $cell->setAlignment('center');
                });
                $i++;
                $sheet->mergeCells('A'.$i.':E'.$i);
                $sheet->cell('A'.$i, function($cell) {
                    $cell->setValue('Powered By WORKSPACE INFOTECH LIMITED');  
                    $cell->setBackground('#33CCCC');  
                    $cell->setAlignment('center');
                });
            });
       })->download('xlsx');


    }

    public function downloadCustomerSampleFile()
    {
        $output= Excel::create('sample-customers', function($excel){
            $excel->sheet('sheet1', function($sheet)
            {
                $sheet->setColumnFormat(array(
                    'A' => '0',
                ));

                /**Header */
                $sheet->cell('A1', function($cell) {$cell->setValue('Phone Number');  $cell->setFontWeight('bold');});
                $sheet->cell('B1', function($cell) {$cell->setValue('Gender');  $cell->setFontWeight('bold');});
                $sheet->cell('C1', function($cell) {$cell->setValue('Name');  $cell->setFontWeight('bold');});
                $sheet->cell('D1', function($cell) {$cell->setValue('Email'); $cell->setFontWeight('bold');  });
                $sheet->cell('E1', function($cell) {$cell->setValue('Club Account'); $cell->setFontWeight('bold');  });
                $sheet->cell('F1', function($cell) {$cell->setValue('WP Availability'); $cell->setFontWeight('bold');  });

                /**Sample Value */
                $sheet->cell('A2', function($cell) {$cell->setValue('880160000000');});
                $sheet->cell('B2', function($cell) {$cell->setValue('Mr');});
                $sheet->cell('C2', function($cell) {$cell->setValue('johnson');});
                $sheet->cell('D2', function($cell) {$cell->setValue('johnson@mail.com');});
                $sheet->cell('E2', function($cell) {$cell->setValue('123456');});
                $sheet->cell('F2', function($cell) {$cell->setValue('1');});

                $sheet->cell('A3', function($cell) {$cell->setValue('880160000001');});
                $sheet->cell('B3', function($cell) {$cell->setValue('Ms');});
                $sheet->cell('C3', function($cell) {$cell->setValue('alena');});
                $sheet->cell('D3', function($cell) {$cell->setValue('alena@mail.com');});
                $sheet->cell('E3', function($cell) {$cell->setValue('985632');});
                $sheet->cell('F3', function($cell) {$cell->setValue('0');});

                $sheet->cell('A4', function($cell) {$cell->setValue('880160000002');});
                $sheet->cell('B4', function($cell) {$cell->setValue('Mrs');});
                $sheet->cell('C4', function($cell) {$cell->setValue('alexa');});
                $sheet->cell('D4', function($cell) {$cell->setValue('alexa@mail.com');});
                $sheet->cell('E4', function($cell) {$cell->setValue('569872');});
                $sheet->cell('F4', function($cell) {$cell->setValue('1');});
            });
       })->download('xlsx');
    }

    public function importCustomers()
    {
        return view('customers.import-customers');
    }

    public function saveImportCustomers(Request $request)
    {
        
        $this->validate(request(), [
            'file' => 'required',
        ]);


        $file_extension = Input::file('import_numbers')->getClientOriginalExtension();
       
        $supportedExt = array('csv', 'xls', 'xlsx');
   
        if (!in_array(strtolower($file_extension), $supportedExt)) {
            return redirect()->back()->withInput($request->all())->with([
                'message' => language_data('Insert Valid Excel or CSV file', Auth::guard('client')->user()->lan_id),
                'message_important' => true
            ]);
        }
        
        $path = $request->file('import_numbers')->getRealPath();
       


        $exist_numb_list = [];
        $allCustomers = [];
        $allCustomerGroupContacts=[];

        $customerGroup = $request->customer_group;
        $broadcastGroup = $request->broadcast_group;



        Excel::filter('chunk')->load($path)->chunk(250, function($results) use ($customerGroup, $broadcastGroup, $exist_numb_list, $allCustomers,$allCustomerGroupContacts )
        {
            foreach($results as $row)
            {
            if($row->phone_number!=''){
                $userPhoneNumber = preg_replace('/^880/', '', (int)$row->phone_number);

            $customer_number = DB::table('customers')
                ->where('number', '=', $userPhoneNumber)
                ->first();

            if (is_null($customer_number)) {
                $customer = new Customer;
                $customer->code = "880";
                $customer->number = $userPhoneNumber;
                $customer->gender = $row->gender;
                $customer->first_name = $row-> name;
                $customer->last_name = "";
                $customer->email = $row->email;
                $customer->user_name = $row->club_account;
                $customer->wp = $row->wp_availability;
                $customer->status = 0;
                $customer->save();

                $customerId = $customer->id;
                
                array_push($allCustomers,$customerId);


                if($customerGroup!=''){
                    $checkCustomerGroup = CustomerGroupCustomer::where('customer_group_id',$customerGroup)->where('customer_id',$customerId)->first();
                        if($checkCustomerGroup==''){
                            $customerGroupContacts = new CustomerGroupCustomer;
                            $customerGroupContacts->customer_group_id = $customerGroup;
                            $customerGroupContacts->customer_id = $customerId;
                            
                            if($row->wp_availability==1){
                                $customerGroupContacts->group_broadcast_id = $broadcastGroup;
                            }else{
                                $customerGroupContacts->group_broadcast_id = null;
                            }
                           
                            $allCustomerGroupContacts[] = $customerGroupContacts->attributesToArray();
                        }
                }
                
            } else {
                array_push($exist_numb_list, (int)$row->phone_number);
                $customerId = $customer_number->id;
                if($customerGroup!=''){
                    $checkCustomerGroup = CustomerGroupCustomer::where('customer_group_id',$customerGroup)->where('customer_id',$customerId)->first();
                        if($checkCustomerGroup==''){
                            $customerGroupContacts = new CustomerGroupCustomer;
                            $customerGroupContacts->customer_group_id = $customerGroup;
                            $customerGroupContacts->customer_id = $customerId;

                            if($row->wp_availability==1){
                                $customerGroupContacts->group_broadcast_id = $broadcastGroup;
                            }else{
                                $customerGroupContacts->group_broadcast_id = null;
                            }

                            $allCustomerGroupContacts[] = $customerGroupContacts->attributesToArray();
                        }
                }
            }

            if(count($allCustomerGroupContacts)==100){
                CustomerGroupCustomer::insert($allCustomerGroupContacts);
                $allCustomerGroupContacts = [];
            }
            }    
               
            }

            if(count($allCustomerGroupContacts)>0){
                CustomerGroupCustomer::insert($allCustomerGroupContacts);
            }
            
            if(count($allCustomers)<=0 && count($allCustomerGroupContacts)<=0){
                Session::flash('alert-warning', 'No Customer to import.');
            }elseif(count($allCustomers)<=0 && count($allCustomerGroupContacts)>0){
                Session::flash('alert-success', 'Group Customer(s) imported successfully');
            }elseif(count($allCustomers)>0 && count($allCustomerGroupContacts)>0 && count($exist_numb_list)<=0){
                Session::flash('alert-success', 'All Customer(s) imported successfully.');
            }
            
            if(count($exist_numb_list)>0){
                Session::flash('alert-warning', 'Some number did not import for duplicacy of phone number or club account '.implode(",", $exist_numb_list));
            }
            
            
        });
        return redirect('/customers-groups');
        
    }











    public function updateWPAvailability(Request $request)
    {
        
        $this->validate(request(), [
            'file' => 'required',
        ]);


        $file_extension = Input::file('import_numbers')->getClientOriginalExtension();
       
        $supportedExt = array('csv', 'xls', 'xlsx');

        
        $path = $request->file('import_numbers')->getRealPath();
       

        




        Excel::filter('chunk')->load($path)->chunk(250, function($results)
        {
            
            foreach($results as $row)
            {
            if($row->phone_number!=''){
                $userPhoneNumber = (int)$row->phone_number;
            $customer_number = Customer::where('number', '=', $userPhoneNumber)->first();

            if($customer_number!=''){
                $customer_number->wp = 1;
                $customer_number->save();
            }
            
            }    
               
            }
            
            
        });
        return redirect('/customers-groups');
        
    }



    public function updateCustomerGroupCustomerBroadcast(Request $request)
    {
        
        $this->validate(request(), [
            'file' => 'required',
        ]);


        $file_extension = Input::file('import_numbers')->getClientOriginalExtension();
       
        $supportedExt = array('csv', 'xls', 'xlsx');
        
        $path = $request->file('import_numbers')->getRealPath();

        Excel::filter('chunk')->load($path)->chunk(250, function($results)
        {
            $customerId = [];
            $customerGroupCustomerId = [];
            $groupBroadcastId= 1;
            $groupId= 1;
            foreach($results as $row)
            {

            if($row->phone_number!=''){
                $userPhoneNumber = (int)$row->phone_number;
                $customer_number = Customer::where('number', '=', $userPhoneNumber)->first();
               
            
            if($customer_number!=''){
                $customer_number->wp = 1;
                $customer_number->save();
                array_push($customerId,$customer_number->id);
            }


            $customerGroupCustomer = CustomerGroupCustomer::where('customer_id',$customer_number->id)->where('customer_group_id',$groupId)->first();
            $customerGroupCustomer->group_broadcast_id = $groupBroadcastId;
            $customerGroupCustomer->save();

            array_push($customerGroupCustomerId,$customerGroupCustomer->id);
            }    
            
            }

            echo "<pre>";
            // print_r($customerId); 
            print_r(count($customerId)); 
            print_r(count($customerGroupCustomerId)); 
            die;
        });
        return redirect('/customers-groups');
        
    }
}
