<?php

namespace App\Http\Controllers;

use App\Config;
use App\Http\Requests;
use App\Events\WhatsApp;
use App\Notification;
use App\Models\User;
use Illuminate\Http\Request;
use App\Models\WhatsappClientDevice;
use App\Models\WhatsappCampaign;
use App\Models\WhatsappCampaignContacts;
use App\Models\WhatsappCampaignHistory;
use App\Models\IntCountryCodes;
use App\Models\Customer;
use App\Models\CustomerGroup;
use App\Models\CustomerGroupCustomer;
use App\Models\GroupBroadcast;
use App\Models\WhatsappSmsHistory;
use App\Http\Controllers\WebhookController;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Input;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Response;
use Illuminate\Support\Facades\Session;
use Maatwebsite\Excel\Facades\Excel;
use DateTime;
use DateTimeZone;
use stdClass;

class WhatsappController extends Controller
{
    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct()
    {
        $this->middleware('auth');
        $this->whatsApp = new WhatsApp();
        $this->webhookController = new WebhookController();
    }

    /**
     * Show the application dashboard.
     *
     * @return \Illuminate\Http\Response
     */
    public function sendQuickWhatsapp()
    {

        $client_device = WhatsappClientDevice::where('status', 'completed')->first();
        $country_code = IntCountryCodes::where('country_code', '880')->select('country_code', 'country_name')->get();

        return view('whatsapp.send-quick-whatsapp', compact('client_device', 'country_code'));
    }

    public function check_create_customer($phn, $cc = "N/A", $fn = "", $ln = "", $email = "", $company = "", $username = "", $dob = "")
    {
        $customer_number = DB::table('customers')
            ->where('number', '=', $phn)
            ->first();

        if (is_null($customer_number)) {
            $customer = new Customer;
            $customer->code = "880";
            $customer->number = $phn;
            $customer->first_name = $fn;
            $customer->last_name = "";
            $customer->dob ="";
            $customer->email = "";
            $customer->user_name = "";
            $customer->status = 0;
            $customer->opt_out_db = "no";
            $customer->save();

            $user_id = $customer->id;

        } else {
            $user_id = $customer_number->id;
        }

        return $user_id;
    }

    public function saveSendQuickWhatsapp(Request $request)
    {

        $param["type"] = "send";

        if (request('message_type') == "plain") {
            $this->validate(request(), [
                'country_code' => 'required',
                'phone' => 'required',
                'message' => 'required'
            ]);
        } else {
            $this->validate(request(), [
                'country_code' => 'required',
                'phone' => 'required',
                'message' => 'required',
                'media' => 'required'
            ]);
        }

        if (request('send_type') == "scheduled") {
            $this->validate(request(), [
                'schedule_time' => 'required',
            ]);
        }

        /**Assigned for DB entry for Regular */
        $messageTime = date("Y-m-d H:i");
        $sendType = 'regular';
        $mediaUrl = "";


        /**Format Date Time */
        if($request->send_type=='scheduled' && $request->schedule_time!=''){
            // Convert datetime to Unix timestamp
            $timestamp = strtotime($request->schedule_time);
            // Subtract time from datetime
            $time = $timestamp - (6 * 60 * 60);
       
            // Date and time after subtraction
            $datetime = date("Y-m-d H:i:s", $time);
            $datetimes = explode(" ", $datetime);
            $scheduleTime = $datetimes[0].'T'.$datetimes[1].'Z';
            $schedule = new stdClass();
            $schedule->date = $scheduleTime;

            /**Assigned for DB entry for schedule */
            $messageTime = $datetime;
            $sendType = 'scheduled';
        }


        $clientDeviceInfo = WhatsappClientDevice::where('status', 'completed')->first();

        if ($clientDeviceInfo == "") {
            return redirect()->back()->withInput(Input::all())->with('message', 'You are not allowed to send message. To send message device status should be COMPLETED!');
        }

        $msg_to_send = request('message');

   

        $requestedNumbers = $request->phone;
        $phoneNumbers = explode( ',', $requestedNumbers );


        $customerIds = [];
        foreach ($phoneNumbers as $key => $value) {
           
            $customer_id = $this->check_create_customer(trim($value), request('country_code'));     
            array_push($customerIds, $customer_id);
        }
        
        $customer_info = Customer::whereIn('id', $customerIds)->get();
       
        $userMessage = $msg_to_send;
        $messageResult= [];

        if (request('message_type') == "media") {

            // Handle File Upload
            if ($request->hasFile('media')) {
          
                // Get filename with the extension
                $filenameWithExt = $request->file('media')->getClientOriginalName();
                // Get just filename
                $filename = pathinfo($filenameWithExt, PATHINFO_FILENAME);
     
                // Get just ext
                $extension = $request->file('media')->getClientOriginalExtension();

                // Filename to store
                $fileNameToStore = preg_replace("/\s+/", "", $filename) . '_' . time() . '.' . $extension;
     
                // Upload Image
                // $path = $request->file('media')->storeAs('public/assets', $fileNameToStore);
                $path = $request->file('media')->move('public/assets', $fileNameToStore);
                $movedUrl = $request->root() . '/public/assets/' . $fileNameToStore;
                $movedUrl = str_replace( 'http://', 'https://', $movedUrl );

                $file_data = new stdClass();
                $file_data->url = $movedUrl; // server url
               
                // $file_data->url = 'http://gulshanclub.com/wp-content/uploads/2016/10/logo.png'; // server url
                $file_data->format = "native";
                $myJSON = json_encode($file_data);
                
                $file_upload_result = $this->whatsApp->wassiFileUpload($myJSON);
            
                $result = $file_upload_result;
                $result = json_decode($result);
           

                try {
                    if ($result->success) {
                        
                        $this_file_id_decoded =json_decode($result->data);
                        
                        if(gettype($this_file_id_decoded)=='object'){
                            $this_file_id =$this_file_id_decoded->meta->file;
                        }else{
                            $this_file_id =$this_file_id_decoded[0]->id;
                        }

                        /**Assign for database entry */
                        $mediaUrl = $this_file_id;

                        $mediaFile = new stdClass();
                        $mediaFile->file = $this_file_id;
                        $mediaFileJson = json_encode($mediaFile);

                        
                        foreach ($customer_info as $key => $value) {
                            $phone = "+" . $value->code . $value->number;
            
                            $data = new stdClass();
                            $data->message = $userMessage;
                            $data->phone = $phone;
                            $data->media = json_decode($mediaFileJson);
                            if($request->send_type=='scheduled' && $request->schedule_time!=''){
                                $data->schedule = $schedule;
                            }

                            $myJSON = json_encode($data);
                            $result = $this->whatsApp->post($myJSON);
                            $result = json_decode($result);

                            $result = json_decode($result->data);
               
                            $user_id = Auth::user()->id;

                            if($result->status==400){
                                $messageResponse['customerId'] = $value->id;
                                $messageResponse['messageId'] = "";
                                $messageResponse['message'] = $userMessage;
                                $messageResponse['clientId'] = $user_id;
                                $messageResponse['phone'] = $phone;
                                $messageResponse['type'] = $sendType;
                                $messageResponse['status'] = "failed";
                                $messageResponse['deliveryStatus'] = "failed";
                                $messageResponse['time'] = $messageTime;
                                $messageResponse['mediaUrl'] = $mediaUrl;
                                $messageResponse['failedReason'] = $result->message;
                                $messageResponse['cron_processing_status'] = 1;
                            }else{
                                $messageResponse['customerId'] = $value->id;
                                $messageResponse['messageId'] = $result->id;
                                $messageResponse['message'] = $userMessage;
                                $messageResponse['clientId'] = $user_id;
                                $messageResponse['phone'] = $phone;
                                $messageResponse['type'] = $sendType;
                                $messageResponse['status'] = "queued";
                                $messageResponse['deliveryStatus'] = "queued";
                                $messageResponse['time'] = $messageTime;
                                $messageResponse['mediaUrl'] = $mediaUrl;
                                $messageResponse['failedReason'] = "";
                                $messageResponse['cron_processing_status'] = 1;
                            }


                            array_push($messageResult,$messageResponse);
                        }
                       
                    }
                } catch (exception $e) {
                    $baseUrl = public_path();
                    $fileToRemove = $baseUrl . '/assets/'.$fileNameToStore;
                    unlink($fileToRemove);
                    // return redirect()->back()->with('message', 'Unable to send message due to media file uploaded issue.');
                }
            }
        } else {
            foreach ($customer_info as $key => $value) {
                $phone = "+" . $value->code . $value->number;
            
                $data = new stdClass();
                $data->message = $userMessage;
                $data->phone = $phone;
                if($request->send_type=='scheduled' && $request->schedule_time!=''){
                    $data->schedule = $schedule;
                }
                $myJSON = json_encode($data);

                $result = $this->whatsApp->post($myJSON);
                $result = json_decode($result);
                $result = json_decode($result->data);
               
                $user_id = Auth::user()->id;

                if($result->status==400){
                    $messageResponse['customerId'] = $value->id;
                    $messageResponse['messageId'] = "";
                    $messageResponse['message'] = $userMessage;
                    $messageResponse['clientId'] = $user_id;
                    $messageResponse['phone'] = $phone;
                    $messageResponse['type'] = $sendType;
                    $messageResponse['status'] = "failed";
                    $messageResponse['deliveryStatus'] = "failed";
                    $messageResponse['time'] = $messageTime;
                    $messageResponse['mediaUrl'] = $mediaUrl;
                    $messageResponse['failedReason'] = $result->message;
                    $messageResponse['cron_processing_status'] = 1;
                }else{
                    $messageResponse['customerId'] = $value->id;
                    $messageResponse['messageId'] = $result->id;
                    $messageResponse['message'] = $userMessage;
                    $messageResponse['clientId'] = $user_id;
                    $messageResponse['phone'] = $phone;
                    $messageResponse['type'] = $sendType;
                    $messageResponse['status'] = "queued";
                    $messageResponse['deliveryStatus'] = "queued";
                    $messageResponse['time'] = $messageTime;
                    $messageResponse['mediaUrl'] = $mediaUrl;
                    $messageResponse['failedReason'] = "";
                    $messageResponse['cron_processing_status'] = 1;
                }
               
                array_push($messageResult,$messageResponse);
            }

            // echo "<pre>";
            // print_r($messageResult);
            // die;
        }

        try {
            foreach ($messageResult as $key => $value) {

                $newSms = new WhatsappSmsHistory();
                $newSms->message_id = $value['messageId'];
                $newSms->message = $value['message'];
           
                $newSms->phone = $value['phone'];
                $newSms->client_id = $value['clientId'];
                $newSms->customer_id = $value['customerId'];
                
                $newSms->type = $value['type'];
                $newSms->status = $value['deliveryStatus'];
                $newSms->deliveryStatus = $value['deliveryStatus'];
                $newSms->time = $value['time'];
                $newSms->media_url = $value['mediaUrl'];
                $newSms->failed_reason = $value['failedReason'];
                $newSms->cron_processing_status = 1;
                $newSms->save();
            }
                
                
             
            return redirect('whatsapp-sms-history');
            
        } catch (exception $e) {
            return view('whatsapp.something-wrong');
        }


    }

    public
    function sendBulkWhatsapp()
    {
        $user_id = Auth::user()->id;

        $client_device = WhatsappClientDevice::where('status', 'completed')->first();
        $whatsapp_campaigns = WhatsappCampaign::where('status', 'active')->get();
        $groups = CustomerGroup::all();
//        return $groups;
        return view('whatsapp.send-bulk-whatsapp', compact('client_device', 'groups', 'whatsapp_campaigns'));
    }

    public function saveSendBulkWhatsapp(Request $request)
    {

        $param["type"] = "send-bulk";

        if (request('message_type') == "plain") {
            $this->validate(request(), [
                'campaign_name' => 'required',
                'group_list_id' => 'required',
                'message' => 'required'
            ]);
        } else {
            $this->validate(request(), [
                'campaign_name' => 'required',
                'group_list_id' => 'required',
                'message' => 'required',
                'media' => 'required'
            ]);
        }

        /**Assigned for DB entry for Regular */
        $messageTime = date("Y-m-d H:i");
        $campaignType = 'regular';
        $mediaUrl = "";

        /**Format Date Time */
        if($request->send_type=='scheduled' && $request->schedule_time!=''){
            // Convert datetime to Unix timestamp
            $timestamp = strtotime($request->schedule_time);
            // Subtract time from datetime
            $time = $timestamp - (6 * 60 * 60);

            // Date and time after subtraction
            $datetime = date("Y-m-d H:i:s", $time);
            $datetimes = explode(" ", $datetime);
            $scheduleTime = $datetimes[0].'T'.$datetimes[1].'Z';
            $schedule = new stdClass();
            $schedule->date = $scheduleTime;

            /**Assigned for DB entry for schedule */
            $messageTime = $datetime;
            $campaignType = 'scheduled';
        }

        $result = [];

        if (isset($request->group_list_id) && is_array($request->group_list_id) && count($request->group_list_id)>0) {
            $broadcast = GroupBroadcast::whereIn('group_id', $request->group_list_id)->get();

            // $customerIds = CustomerGroupCustomer::whereIn('customer_group_id', $request->group_list_id)->pluck('customer_id')->toArray();
            // $get_customer_data = Customer::whereIn('id', $customerIds)->get();
        }


        $clientDeviceInfo = WhatsappClientDevice::where('status', 'completed')->first();
        if ($clientDeviceInfo == '') {
            return redirect()->back()->withInput(Input::all())->with('message', 'You are not allowed to send message. To send message device status should be COMPLETED!');
        }

        $msg_to_send = request('message');
        $userMessage = $request->message;
        $contacts = [];
        foreach ($broadcast as $r) {

            $msg_data = array(
                'groupBroadCastId' => $r['id'],
                'broadcast' => $r['broadcast_id'],
                'message' => $userMessage,
                'groupId' => $r['group_id'],
            );
            array_push($contacts, $msg_data);
        }

        $sentMessageResponse=[];

        if (request('message_type') == "media") {
            // Handle File Upload
            if ($request->hasFile('media')) {
                // Get filename with the extension
                $filenameWithExt = $request->file('media')->getClientOriginalName();
                // Get just filename
                $filename = pathinfo($filenameWithExt, PATHINFO_FILENAME);

                // Get just ext
                $extension = $request->file('media')->getClientOriginalExtension();
                // Filename to store
                $fileNameToStore = preg_replace("/\s+/", "", $filename) . '_' . time() . '.' . $extension;

                // Upload Image
                // $path = $request->file('media')->storeAs('public/assets', $fileNameToStore);
                $path = $request->file('media')->move('public/assets', $fileNameToStore);

                $this_file_id = $fileNameToStore;

                /**Assign for database entry */
                $mediaUrl = $this_file_id;
            }
        }
        foreach ($contacts as $key => $value) {
            $user_id = Auth::user()->id;
            $messageResponse['customerId'] = "";
            $messageResponse['group_id'] = $value['groupId'];
            $messageResponse['group_broadcast_id'] = $value['groupBroadCastId'];
            $messageResponse['broadcast_id'] = $value['broadcast'];
            $messageResponse['messageId'] = "";
            $messageResponse['message'] = $value['message'];
            $messageResponse['clientId'] = $user_id;
            $messageResponse['phone'] = "";
            $messageResponse['type'] = $campaignType;
            $messageResponse['status'] = "queued";
            $messageResponse['deliveryStatus'] = "queued";
            $messageResponse['time'] = $messageTime;
            $messageResponse['mediaUrl'] = $mediaUrl;
            $messageResponse['failedReason'] = "";
            array_push($sentMessageResponse,$messageResponse);
        }

        try {
            $user_id = Auth::user()->id;

            $newCampaign = new WhatsappCampaignHistory();
            $newCampaign->campaign_name = request('campaign_name');
            $newCampaign->client_id = $user_id;
            $newCampaign->message = request('message');
            $newCampaign->type = $campaignType;
            $newCampaign->time = $messageTime;
            $newCampaign->media_url = $mediaUrl;
            $success_campaign = $newCampaign->save();
            $campaignId = $newCampaign->id;
            $newCampaign_name = $newCampaign->campaign_name;

            $allCampaignSms = [];

            foreach ($sentMessageResponse as $contact) { //$intersts array contains input data

                $user_id = Auth::user()->id;
                $newSms = new WhatsappSmsHistory();
                $newSms->campaign_id = $campaignId;
                $newSms->message_id = $contact['messageId'];
                $newSms->group_id = $contact['group_id'];
                $newSms->group_broadcast_id = $contact['group_broadcast_id'];
                $newSms->broadcast_id = $contact['broadcast_id'];
                $newSms->message = $contact['message'];
                $newSms->phone = $contact['phone'];
                $newSms->client_id = $contact['clientId'];
                $newSms->customer_id = $contact['customerId'];
                $newSms->type = $contact['type'];
                $newSms->status = $contact['status'];
                $newSms->deliveryStatus = $contact['deliveryStatus'];
                $newSms->time = $contact['time'];
                $newSms->media_url = $contact['mediaUrl'];
                $newSms->failed_reason = $contact['failedReason'];

                array_push($allCampaignSms,$newSms->attributesToArray());

                if(count($allCampaignSms)==100){
                    WhatsappSmsHistory::insert($allCampaignSms);
                    $allCampaignSms = [];
                }
            }

            if(count($allCampaignSms)>0){
                WhatsappSmsHistory::insert($allCampaignSms);
            }

            if($success_campaign){
                $user_infos = User::where('user_type', 'administrator')->get();
                if(!empty($user_infos)){
                    foreach($user_infos as $user_info){
                        $user_id = $user_info->id;
                        $new_notification = new Notification();
                        $new_notification->user_id = $user_id;
                        $new_notification->type = "";
                        $new_notification->icon_class = "fas fa-envelope mr-2";
                        $new_notification->link = "javascript:void(0)";
                        $new_notification->title = $newCampaign_name;
                        $new_notification->message = "New Bulk Whatsapp Broadcast Message";
                        $new_notification->is_read = 0;
                        $new_notification->save();
                    }
                }
                $config_info = Config::first();
                $subject = "New ".$newCampaign_name." Bulk Whatsapp Broadcast Message";
                if(!empty($config_info)){
                    $config_mail = $config_info->fixed_email;
                    $config_contact_number = $config_info->fixed_contact_number;
                    $fixed_contact_number_content = $config_info->fixed_contact_number_content;
                    Mail::send('mailTemplate.whatsapp_message_mail_sending', ['newCampaign_name' =>$newCampaign_name],
                        function ($message) use ($config_mail, $subject) {
                            $message->to([$config_mail])->subject($subject);
                        });

                    $data = new stdClass();
                    $data->message = $fixed_contact_number_content;
                    $data->phone = $config_contact_number;
                    $myJSON = json_encode($data);
                    $result = $this->whatsApp->post($myJSON);
                    $result = json_decode($result);
                    $result = json_decode($result->data);
                    Log::info("Broadcast Whatsapp Message Config Result: ".json_encode($result));

                }
            }

            return redirect('whatsapp-bulk-sms-history');

        } catch (exception $e) {
            return view('whatsapp.something-wrong');
        }


    }

    public function saveSendBulkWhatsappPreviousWassi(Request $request)
    {

        $param["type"] = "send-bulk";

        if (request('message_type') == "plain") {
            $this->validate(request(), [
                'campaign_name' => 'required',
                'group_list_id' => 'required',
                'message' => 'required'
            ]);
        } else {
            $this->validate(request(), [
                'campaign_name' => 'required',
                'group_list_id' => 'required',
                'message' => 'required',
                'media' => 'required'
            ]);
        }

        /**Assigned for DB entry for Regular */
        $messageTime = date("Y-m-d H:i");
        $campaignType = 'regular';
        $mediaUrl = "";

        /**Format Date Time */
        if($request->send_type=='scheduled' && $request->schedule_time!=''){
            // Convert datetime to Unix timestamp
            $timestamp = strtotime($request->schedule_time);
            // Subtract time from datetime
            $time = $timestamp - (6 * 60 * 60);
            
            // Date and time after subtraction
            $datetime = date("Y-m-d H:i:s", $time);
            $datetimes = explode(" ", $datetime);
            $scheduleTime = $datetimes[0].'T'.$datetimes[1].'Z';
            $schedule = new stdClass();
            $schedule->date = $scheduleTime;

            /**Assigned for DB entry for schedule */
            $messageTime = $datetime;
            $campaignType = 'scheduled';
        }

        $result = [];
        
        if (isset($request->group_list_id) && is_array($request->group_list_id) && count($request->group_list_id)>0) {
            $broadcast = GroupBroadcast::whereIn('group_id', $request->group_list_id)->get();

            // $customerIds = CustomerGroupCustomer::whereIn('customer_group_id', $request->group_list_id)->pluck('customer_id')->toArray();
            // $get_customer_data = Customer::whereIn('id', $customerIds)->get();
        }


        $clientDeviceInfo = WhatsappClientDevice::where('status', 'completed')->first();
        if ($clientDeviceInfo == '') {
            return redirect()->back()->withInput(Input::all())->with('message', 'You are not allowed to send message. To send message device status should be COMPLETED!');
        }

        $msg_to_send = request('message');
        $userMessage = $request->message;
        $contacts = [];
        foreach ($broadcast as $r) {

            $msg_data = array(
                'groupBroadCastId' => $r['id'],
                'broadcast' => $r['broadcast_id'],
                'message' => $userMessage,
                'groupId' => $r['group_id'],
            );
            array_push($contacts, $msg_data);
        }

        $sentMessageResponse=[];

        if (request('message_type') == "media") {
            // Handle File Upload
            if ($request->hasFile('media')) {
                // Get filename with the extension
                $filenameWithExt = $request->file('media')->getClientOriginalName();
                // Get just filename
                $filename = pathinfo($filenameWithExt, PATHINFO_FILENAME);
     
                // Get just ext
                $extension = $request->file('media')->getClientOriginalExtension();
                // Filename to store
                $fileNameToStore = preg_replace("/\s+/", "", $filename) . '_' . time() . '.' . $extension;
     
                // Upload Image
                // $path = $request->file('media')->storeAs('public/assets', $fileNameToStore);
                $path = $request->file('media')->move('public/assets', $fileNameToStore);
              
                $movedUrl = $request->root() . '/public/assets/' . $fileNameToStore;
                $movedUrl = str_replace( 'http://', 'https://', $movedUrl );

                $file_data = new stdClass();
                $file_data->url = $movedUrl; // server url
                
                // $file_data->url = 'http://gulshanclub.com/wp-content/uploads/2016/10/logo.png'; // server url
                $file_data->format = "native";
                $myJSON = json_encode($file_data);
                
                $file_upload_result = $this->whatsApp->wassiFileUpload($myJSON);
            
                $result = $file_upload_result;
                $result = json_decode($result);
                
                
                try {
                    if ($result->success) {
                        
                        $this_file_id_decoded =json_decode($result->data);
                        
                        if(gettype($this_file_id_decoded)=='object'){
                            if($this_file_id_decoded->status && $this_file_id_decoded->status=='400'){
                                Session::flash('alert-danger', $this_file_id_decoded->message);

                                return redirect('send-schedule-whatsapp');
                            }

                            $this_file_id =$this_file_id_decoded->meta->file;
                        }else{
                            $this_file_id =$this_file_id_decoded[0]->id;
                        }

                        /**Assign for database entry */
                        $mediaUrl = $this_file_id;
                        
                        foreach ($contacts as $key => $value) {
                            $user_id = Auth::user()->id;
                            $messageResponse['customerId'] = "";
                            $messageResponse['group_id'] = $value['groupId'];
                            $messageResponse['group_broadcast_id'] = $value['groupBroadCastId'];
                            $messageResponse['broadcast_id'] = $value['broadcast'];
                            $messageResponse['messageId'] = "";
                            $messageResponse['message'] = $value['message'];
                            $messageResponse['clientId'] = $user_id;
                            $messageResponse['phone'] = "";
                            $messageResponse['type'] = $campaignType;
                            $messageResponse['status'] = "queued";
                            $messageResponse['deliveryStatus'] = "queued";
                            $messageResponse['time'] = $messageTime;
                            $messageResponse['mediaUrl'] = $mediaUrl;
                            $messageResponse['failedReason'] = "";
                            array_push($sentMessageResponse,$messageResponse);
                        }
                    }
                } catch (exception $e) {
                    $baseUrl = public_path();
                    $fileToRemove = $baseUrl . '/assets/'.$fileNameToStore;
                    unlink($fileToRemove);
                    return redirect()->back()->with('message', 'Unable to send message due to media file uploaded issue.');
                }
            }
        } else {
            foreach ($contacts as $key => $value) {
                $user_id = Auth::user()->id;
                $messageResponse['customerId'] = "";
                $messageResponse['group_id'] = $value['groupId'];
                $messageResponse['group_broadcast_id'] = $value['groupBroadCastId'];
                $messageResponse['broadcast_id'] = $value['broadcast'];
                $messageResponse['messageId'] = "";
                $messageResponse['message'] = $value['message'];
                $messageResponse['clientId'] = $user_id;
                $messageResponse['phone'] = "";
                $messageResponse['type'] = $campaignType;
                $messageResponse['status'] = "queued";
                $messageResponse['deliveryStatus'] = "queued";
                $messageResponse['time'] = $messageTime;
                $messageResponse['mediaUrl'] = $mediaUrl;
                $messageResponse['failedReason'] = "";
                array_push($sentMessageResponse,$messageResponse);
            }
        }

        try {
                $user_id = Auth::user()->id;

                $newCampaign = new WhatsappCampaignHistory();
                $newCampaign->campaign_name = request('campaign_name');
                $newCampaign->client_id = $user_id;
                $newCampaign->message = request('message');
                $newCampaign->type = $campaignType;
                $newCampaign->time = $messageTime;
                $newCampaign->media_url = $mediaUrl;
                $newCampaign->save();
                $campaignId = $newCampaign->id;

                $allCampaignSms = [];
                
                foreach ($sentMessageResponse as $contact) { //$intersts array contains input data
     
                    $user_id = Auth::user()->id;
                    $newSms = new WhatsappSmsHistory();
                    $newSms->campaign_id = $campaignId;
                    $newSms->message_id = $contact['messageId'];
                    $newSms->group_id = $contact['group_id'];
                    $newSms->group_broadcast_id = $contact['group_broadcast_id'];
                    $newSms->broadcast_id = $contact['broadcast_id'];
                    $newSms->message = $contact['message'];
                    $newSms->phone = $contact['phone'];
                    $newSms->client_id = $contact['clientId'];
                    $newSms->customer_id = $contact['customerId'];
                    $newSms->type = $contact['type'];
                    $newSms->status = $contact['status'];
                    $newSms->deliveryStatus = $contact['deliveryStatus'];
                    $newSms->time = $contact['time'];
                    $newSms->media_url = $contact['mediaUrl'];
                    $newSms->failed_reason = $contact['failedReason'];

                    array_push($allCampaignSms,$newSms->attributesToArray());
                    
                    if(count($allCampaignSms)==100){
                        WhatsappSmsHistory::insert($allCampaignSms);
                        $allCampaignSms = [];
                    }
                }
     
                if(count($allCampaignSms)>0){
                    WhatsappSmsHistory::insert($allCampaignSms);
                }
                

                 $this->webhookController->manageUnsentMessage();

                return redirect('whatsapp-bulk-sms-history');
            
        } catch (exception $e) {
            return view('whatsapp.something-wrong');
        }


    }
    
    // public function saveSendBulkWhatsapp(Request $request)
    // {

    //     $param["type"] = "send-bulk";

    //     if (request('message_type') == "plain") {
    //         $this->validate(request(), [
    //             'campaign_name' => 'required',
    //             'group_list_id' => 'required',
    //             'message' => 'required'
    //         ]);
    //     } else {
    //         $this->validate(request(), [
    //             'campaign_name' => 'required',
    //             'group_list_id' => 'required',
    //             'message' => 'required',
    //             'media' => 'required'
    //         ]);
    //     }

    //     /**Assigned for DB entry for Regular */
    //     $messageTime = date("Y-m-d H:i");
    //     $campaignType = 'regular';
    //     $mediaUrl = "";

    //     /**Format Date Time */
    //     if($request->send_type=='scheduled' && $request->schedule_time!=''){
    //         // Convert datetime to Unix timestamp
    //         $timestamp = strtotime($request->schedule_time);
    //         // Subtract time from datetime
    //         $time = $timestamp - (6 * 60 * 60);
            
    //         // Date and time after subtraction
    //         $datetime = date("Y-m-d H:i:s", $time);
    //         $datetimes = explode(" ", $datetime);
    //         $scheduleTime = $datetimes[0].'T'.$datetimes[1].'Z';
    //         $schedule = new stdClass();
    //         $schedule->date = $scheduleTime;

    //         /**Assigned for DB entry for schedule */
    //         $messageTime = $datetime;
    //         $campaignType = 'scheduled';
    //     }

    //     $result = [];

    //     if (isset($request->group_list_id) && is_array($request->group_list_id) && count($request->group_list_id)>0) {
    //         $customerIds = CustomerGroupCustomer::whereIn('customer_group_id', $request->group_list_id)->pluck('customer_id')->toArray();
    //         $get_customer_data = Customer::whereIn('id', $customerIds)->get();
    //     }


    //     $clientDeviceInfo = WhatsappClientDevice::where('status', 'completed')->first();
    //     if ($clientDeviceInfo == '') {
    //         return redirect()->back()->withInput(Input::all())->with('message', 'You are not allowed to send message. To send message device status should be COMPLETED!');
    //     }

    //     $msg_to_send = request('message');
       
    //     $contacts = [];
    //     foreach ($get_customer_data as $r) {
    //         $userMessage = $request->message;
    //         $userMessage = str_replace("<%Email_Address%>", $r['email'], $userMessage);
    //         $userMessage = str_replace("<%Phone_Number%>", $r['code'] . $r['number'], $userMessage);
    //         $userMessage = str_replace("<%Club_Account%>", $r['user_name'], $userMessage);
    //         $userMessage = str_replace("<%First_Name%>", $r['first_name'], $userMessage);
    //         $userMessage = str_replace("<%Gender%>", $r['gender'], $userMessage);

    //         $msg_data = array(
    //             'contactNumber' => "+" . $r['code'] . $r['number'],
    //             'message' => $userMessage,
    //             'customerId' => $r['id'],
    //         );
    //         array_push($contacts, $msg_data);
    //     }

    //     $sentMessageResponse=[];

    //     if (request('message_type') == "media") {
    //         // Handle File Upload
    //         if ($request->hasFile('media')) {
    //             // Get filename with the extension
    //             $filenameWithExt = $request->file('media')->getClientOriginalName();
    //             // Get just filename
    //             $filename = pathinfo($filenameWithExt, PATHINFO_FILENAME);
     
    //             // Get just ext
    //             $extension = $request->file('media')->getClientOriginalExtension();
    //             // Filename to store
    //             $fileNameToStore = preg_replace("/\s+/", "", $filename) . '_' . time() . '.' . $extension;
     
    //             // Upload Image
    //             // $path = $request->file('media')->storeAs('public/assets', $fileNameToStore);
    //             $path = $request->file('media')->move('public/assets', $fileNameToStore);
              
    //             $movedUrl = $request->root() . '/public/assets/' . $fileNameToStore;
    //             $movedUrl = str_replace( 'http://', 'https://', $movedUrl );

    //             $file_data = new stdClass();
    //             $file_data->url = $movedUrl; // server url
                
    //             // $file_data->url = 'http://gulshanclub.com/wp-content/uploads/2016/10/logo.png'; // server url
    //             $file_data->format = "native";
    //             $myJSON = json_encode($file_data);
                
    //             $file_upload_result = $this->whatsApp->wassiFileUpload($myJSON);
            
    //             $result = $file_upload_result;
    //             $result = json_decode($result);
                
                
    //             try {
    //                 if ($result->success) {
                        
    //                     $this_file_id_decoded =json_decode($result->data);
                        
    //                     if(gettype($this_file_id_decoded)=='object'){
    //                         if($this_file_id_decoded->status && $this_file_id_decoded->status=='400'){
    //                             Session::flash('alert-danger', $this_file_id_decoded->message);

    //                             return redirect('send-schedule-whatsapp');
    //                         }

    //                         $this_file_id =$this_file_id_decoded->meta->file;
    //                     }else{
    //                         $this_file_id =$this_file_id_decoded[0]->id;
    //                     }

    //                     /**Assign for database entry */
    //                     $mediaUrl = $this_file_id;
                        
    //                     foreach ($contacts as $key => $value) {
    //                         $user_id = Auth::user()->id;
    //                         $messageResponse['customerId'] = $value['customerId'];
    //                         $messageResponse['messageId'] = "";
    //                         $messageResponse['message'] = $value['message'];
    //                         $messageResponse['clientId'] = $user_id;
    //                         $messageResponse['phone'] = $value['contactNumber'];
    //                         $messageResponse['type'] = $campaignType;
    //                         $messageResponse['status'] = "queued";
    //                         $messageResponse['deliveryStatus'] = "queued";
    //                         $messageResponse['time'] = $messageTime;
    //                         $messageResponse['mediaUrl'] = $mediaUrl;
    //                         $messageResponse['failedReason'] = "";
    //                         array_push($sentMessageResponse,$messageResponse);
    //                     }
    //                 }
    //             } catch (exception $e) {
    //                 $baseUrl = public_path();
    //                 $fileToRemove = $baseUrl . '/assets/'.$fileNameToStore;
    //                 unlink($fileToRemove);
    //                 return redirect()->back()->with('message', 'Unable to send message due to media file uploaded issue.');
    //             }
    //         }
    //     } else {
    //         foreach ($contacts as $key => $value) {
    //             $user_id = Auth::user()->id;
    //             $messageResponse['customerId'] = $value['customerId'];
    //             $messageResponse['messageId'] = "";
    //             $messageResponse['message'] = $value['message'];
    //             $messageResponse['clientId'] = $user_id;
    //             $messageResponse['phone'] = $value['contactNumber'];
    //             $messageResponse['type'] = $campaignType;
    //             $messageResponse['status'] = "queued";
    //             $messageResponse['deliveryStatus'] = "queued";
    //             $messageResponse['time'] = $messageTime;
    //             $messageResponse['mediaUrl'] = $mediaUrl;
    //             $messageResponse['failedReason'] = "";
    //             array_push($sentMessageResponse,$messageResponse);
    //         }
    //     }

    //     try {
    //             $user_id = Auth::user()->id;

    //             $newCampaign = new WhatsappCampaignHistory();
    //             $newCampaign->campaign_name = request('campaign_name');
    //             $newCampaign->client_id = $user_id;
    //             $newCampaign->message = request('message');
    //             $newCampaign->type = $campaignType;
    //             $newCampaign->time = $messageTime;
    //             $newCampaign->media_url = $mediaUrl;
    //             $newCampaign->save();
    //             $campaignId = $newCampaign->id;

    //             $allCampaignSms = [];
                
    //             foreach ($sentMessageResponse as $contact) { //$intersts array contains input data
     
    //                 $user_id = Auth::user()->id;
    //                 $newSms = new WhatsappSmsHistory();
    //                 $newSms->campaign_id = $campaignId;
    //                 $newSms->message_id = $contact['messageId'];
    //                 $newSms->message = $contact['message'];
    //                 $newSms->phone = $contact['phone'];
    //                 $newSms->client_id = $contact['clientId'];
    //                 $newSms->customer_id = $contact['customerId'];
    //                 $newSms->type = $contact['type'];
    //                 $newSms->status = $contact['status'];
    //                 $newSms->deliveryStatus = $contact['deliveryStatus'];
    //                 $newSms->time = $contact['time'];
    //                 $newSms->media_url = $contact['mediaUrl'];
    //                 $newSms->failed_reason = $contact['failedReason'];

    //                 array_push($allCampaignSms,$newSms->attributesToArray());
                    
    //                 if(count($allCampaignSms)==100){
    //                     WhatsappSmsHistory::insert($allCampaignSms);
    //                     $allCampaignSms = [];
    //                 }
    //             }
     
    //             if(count($allCampaignSms)>0){
    //                 WhatsappSmsHistory::insert($allCampaignSms);
    //             }
                


    //             return redirect('whatsapp-sms-history');
            
    //     } catch (exception $e) {
    //         return view('whatsapp.something-wrong');
    //     }


    // }




    public function whatsappSmsHistory(Request $request)
    {
        $limit = 10;
        $search_phone_number = '';
        $search_customer_group = [];
        $search_campaign = [];
        $search_msg_type = "";
        $search_date_from = "";
        $search_date_to = "";
        $search_msg_status = "";
        $search_msg_delivery_status = "";

        $customerGroup = CustomerGroup::orderBy('created_at', 'desc')->get();
        $campaignList = WhatsappCampaignHistory::orderBy('created_at', 'desc')->get();
        
        
        if($request->phone_number || $request->customer_group || $request->campaigns || $request->date_from 
        ||$request->date_to || $request->msg_delivery_status || $request->msg_status){
            
            $search_phone_number = $request->phone_number;
            $search_user_name = $request->user_name;
            $search_code = $request->code;
            if($request->customer_group){
                $search_customer_group = $request->customer_group;
            }
            
            if($request->campaigns){
                $search_campaign = $request->campaigns;
            }

            if($request->msg_type){
                $search_msg_type = $request->msg_type;
            }
            if($request->date_from){
                $search_date_from = $request->date_from;
            }
            
            if($request->date_to){
                $search_date_to = $request->date_to;
            }
            
            if($request->msg_status){
                $search_msg_status = $request->msg_status;
            }
            
            if($request->msg_delivery_status){
                $search_msg_delivery_status = $request->msg_delivery_status;
            }
            
            
            $smsQuery = WhatsappSmsHistory::query();
            $smsQuery->where('broadcast_id',null);
            if ($request->phone_number != '') {
                $smsQuery->where('phone', 'like', '%' . $request->phone_number . '%');
            }

            if($request->date_from || $request->date_to){
                $format_date_from = date('Y-m-d H:i:s', strtotime($search_date_from));
                $format_date_to = date('Y-m-d 23:59:59', strtotime($search_date_to));

                if ($search_date_from != '' && $search_date_to != '') {
                    $smsQuery->whereBetween('created_at', [$format_date_from, $format_date_to]);
                }
                if ($search_date_from != '' && $search_date_to == '') {
                    $smsQuery->where('created_at', '>=', $format_date_from);
                }
                if ($search_date_from == '' && $search_date_to != '') {
                    $smsQuery->where('created_at', '<=', $format_date_to);
                }
            }

            if($search_customer_group){
                $customerId = CustomerGroupCustomer::whereIn('customer_group_id', $search_customer_group)->pluck('customer_id')->toArray();
               
                $smsQuery->whereIn('customer_id',$customerId);
            }
            
            if($search_campaign){
                $smsQuery->whereIn('campaign_id',$search_campaign);
            }

            if ($search_msg_type != "") {
                $smsQuery->where('type', $search_msg_type);
            }
            
            if ($search_msg_status != "") {
                $smsQuery->where('status', $search_msg_status);
            }
            
            if ($search_msg_delivery_status != "") {
                $smsQuery->where('deliveryStatus', $search_msg_delivery_status);
            }

            $smsHistory = $smsQuery->orderBy('created_at','Desc')->with('campaign')->paginate($limit);
           
        }else{
            $smsHistory = WhatsappSmsHistory::where('broadcast_id',null)->orderBy('created_at','Desc')->with('campaign','customer','group')->paginate($limit);
        }
      
        // return $smsHistory;

        $data["smsHistory"] = $smsHistory;
        $data["customerGroup"] = $customerGroup;
        $data["campaignList"] = $campaignList;
        $data["page"] = $smsHistory->currentPage();
        $data["limit"] = $limit;
        $data["type"] = $search_msg_type;
        $data["date_from"] = $search_date_from;
        $data["date_to"] = $search_date_to;
        $data["status"] = $search_msg_status;
        $data["deliveryStatus"] = $search_msg_delivery_status;
        $data["search_customer_group"] = $search_customer_group;
        $data["search_campaign"] = $search_campaign;
        $data["search_phone_number"] = $search_phone_number;
        $data["search_customer_group_string"] = $search_customer_group && count($search_customer_group)>0 ? implode(",",$search_customer_group):'';
        $data["search_campaign_string"] = $search_campaign && count($search_campaign)>0 ? implode(",",$search_campaign):'';

        return view('whatsapp.whatsapp-sms-history', $data);
    }

    public function exportAllSmsCsvFile(Request $request)
    {
        $search_phone_number = '';
        $search_customer_group = [];
        $search_campaign = [];
        $search_msg_type = "";
        $search_date_from = "";
        $search_date_to = "";
        $search_msg_status = "";
        $search_msg_delivery_status = "";
        
        if($request->msg_type || $request->campaigns || $request->phone_number || $request->customer_group || $request->date_from 
        ||$request->date_to || $request->msg_delivery_status || $request->msg_status){
            
            $search_phone_number = $request->phone_number;
            $search_user_name = $request->user_name;
            $search_code = $request->code;
            
            if($request->customer_group){
                $search_customer_group =  explode (",", $request->customer_group);
            }

            if($request->campaigns){
                $search_campaign = explode (",", $request->campaigns);
            }

            if($request->msg_type){
                $search_msg_type = $request->msg_type;
            }
            if($request->date_from){
                $search_date_from = $request->date_from;
            }
            
            if($request->date_to){
                $search_date_to = $request->date_to;
            }
            
            if($request->msg_status){
                $search_msg_status = $request->msg_status;
            }
            
            if($request->msg_delivery_status){
                $search_msg_delivery_status = $request->msg_delivery_status;
            }
            
            
            $smsQuery = WhatsappSmsHistory::query();
            
            if ($request->phone_number != '') {
                $smsQuery->where('phone', 'like', '%' . $request->phone_number . '%');
            }

            if($request->date_from || $request->date_to){
                $format_date_from = date('Y-m-d H:i:s', strtotime($search_date_from));
                $format_date_to = date('Y-m-d 23:59:59', strtotime($search_date_to));

                if ($search_date_from != '' && $search_date_to != '') {
                    $smsQuery->whereBetween('created_at', [$format_date_from, $format_date_to]);
                }
                if ($search_date_from != '' && $search_date_to == '') {
                    $smsQuery->where('created_at', '>=', $format_date_from);
                }
                if ($search_date_from == '' && $search_date_to != '') {
                    $smsQuery->where('created_at', '<=', $format_date_to);
                }
            }

            if($search_customer_group){
                $customerId = CustomerGroupCustomer::whereIn('customer_group_id', $search_customer_group)->pluck('customer_id')->toArray();
               
                $smsQuery->whereIn('customer_id',$customerId);
            }

            if($search_campaign){
                $smsQuery->whereIn('campaign_id',$search_campaign);
            }

            if ($search_msg_type != "") {
                $smsQuery->where('type', $search_msg_type);
            }
            
            if ($search_msg_status != "") {
                $smsQuery->where('status', $search_msg_status);
            }
            
            if ($search_msg_delivery_status != "") {
                $smsQuery->where('deliveryStatus', $search_msg_delivery_status);
            }

            $smsQuery->where('broadcast_id',null);

            $smsHistory = $smsQuery->orderBy('created_at','Desc')->with('campaign','customer','broadcast')->get();
           
        }else{
            $smsHistory = WhatsappSmsHistory::where('broadcast_id','!=',null)->orderBy('created_at','Desc')->with('campaign','customer','broadcast')->get();
        }
        
        

        $output= Excel::create('whatsapp-sms-list', function($excel) use ($smsHistory) {
            
            $excel->sheet('sheet1', function($sheet) use ($smsHistory)
            {
                $sheet->setColumnFormat(array(
                    'D' => '0',
                ));
                
                $sheet->mergeCells('A1:H2');
                $sheet->cell('A1', function($cell) {
                    $cell->setValue('GULSHAN CLUB LIMITED');  
                    $cell->setBackground('#33CCCC');  
                    $cell->setAlignment('center');
                    $cell->setFontWeight('bold');    
                    $cell->setFontSize('20');    
                });

                $sheet->mergeCells('A3:H4');
                $sheet->cell('A3', function($cell) use ($smsHistory) {
                    
                    if(count($smsHistory)>0){
                        $count = count($smsHistory);
                        $lastArray = $count-1;
                        $excelFromDate =date('Y/m/d', strtotime($smsHistory[$lastArray]['created_at']));
                        $excelToDate = date('Y/m/d', strtotime($smsHistory[0]['created_at']));
                    }else{
                        $excelFromDate='N/A';
                        $excelToDate = 'N/A';
                    }
                    

                    $cell->setValue('Whatsapp Message List('.$excelFromDate.' to '.$excelToDate.')');  
                    $cell->setBackground('#008686');  
                    $cell->setAlignment('center');
                    $cell->setFontWeight('bold');    
                    $cell->setFontSize('17');    
                });

               
                
                $sheet->cell('A6', function($cell) {$cell->setValue('No.');  $cell->setFontWeight('bold');});
                $sheet->cell('B6', function($cell) {$cell->setValue('Message Type');  $cell->setFontWeight('bold');});
                $sheet->cell('C6', function($cell) {$cell->setValue('Sms created Date'); $cell->setFontWeight('bold');  });
                $sheet->cell('D6', function($cell) {$cell->setValue('To'); $cell->setFontWeight('bold');  });
                $sheet->cell('E6', function($cell) {$cell->setValue('Club Account'); $cell->setFontWeight('bold');  });
                $sheet->cell('F6', function($cell) {$cell->setValue('Message'); $cell->setFontWeight('bold');  });
                $sheet->cell('G6', function($cell) {$cell->setValue('Sms Status'); $cell->setFontWeight('bold');  });
                $sheet->cell('H6', function($cell) {$cell->setValue('Delivery Status'); $cell->setFontWeight('bold');  });

                $i=7;
                
                foreach ($smsHistory as $key => $value) {
                    $sheet->cell('A'.$i, $key+1); 
                    $sheet->cell('B'.$i, $value['type']); 
                    $sheet->cell('C'.$i, date('Y-m-d H:i A', (strtotime($value['created_at'])))); 
                    $sheet->cell('D'.$i, $value['phone']); 
                    $sheet->cell('E'.$i, $value['customer']['user_name']); 
                    $sheet->cell('F'.$i, $value['message']); 
                    $sheet->cell('G'.$i, $value['status']); 
                    $sheet->cell('H'.$i, $value['deliveryStatus']); 
                    $i++;
                }

                $i++;
                $sheet->mergeCells('A'.$i.':H'.$i);
                $sheet->cell('A'.$i, function($cell) {
                    $cell->setValue('This file is software generated');  
                    $cell->setBackground('#6666CC');  
                    $cell->setAlignment('center');
                });
                $i++;
                $sheet->mergeCells('A'.$i.':H'.$i);
                $sheet->cell('A'.$i, function($cell) {
                    $cell->setValue('Powered By WORKSPACE INFOTECH LIMITED');  
                    $cell->setBackground('#33CCCC');  
                    $cell->setAlignment('center');
                });
            });
       })->download('xlsx');
    }



    public function exportAllBulkSmsCsvFile(Request $request)
    {
        $search_phone_number = '';
        $search_customer_group = [];
        $search_campaign = [];
        $search_msg_type = "";
        $search_date_from = "";
        $search_date_to = "";
        $search_msg_status = "";
        $search_msg_delivery_status = "";
        
        if($request->msg_type || $request->campaigns || $request->phone_number || $request->customer_group || $request->date_from 
        ||$request->date_to || $request->msg_delivery_status || $request->msg_status){
            
            $search_phone_number = $request->phone_number;
            $search_user_name = $request->user_name;
            $search_code = $request->code;
            
            if($request->customer_group){
                $search_customer_group =  explode (",", $request->customer_group);
            }

            if($request->campaigns){
                $search_campaign = explode (",", $request->campaigns);
            }

            if($request->msg_type){
                $search_msg_type = $request->msg_type;
            }
            if($request->date_from){
                $search_date_from = $request->date_from;
            }
            
            if($request->date_to){
                $search_date_to = $request->date_to;
            }
            
            if($request->msg_status){
                $search_msg_status = $request->msg_status;
            }
            
            if($request->msg_delivery_status){
                $search_msg_delivery_status = $request->msg_delivery_status;
            }
            
            
            $smsQuery = WhatsappSmsHistory::query();
            
            if ($request->phone_number != '') {
                $smsQuery->where('phone', 'like', '%' . $request->phone_number . '%');
            }

            if($request->date_from || $request->date_to){
                $format_date_from = date('Y-m-d H:i:s', strtotime($search_date_from));
                $format_date_to = date('Y-m-d 23:59:59', strtotime($search_date_to));

                if ($search_date_from != '' && $search_date_to != '') {
                    $smsQuery->whereBetween('created_at', [$format_date_from, $format_date_to]);
                }
                if ($search_date_from != '' && $search_date_to == '') {
                    $smsQuery->where('created_at', '>=', $format_date_from);
                }
                if ($search_date_from == '' && $search_date_to != '') {
                    $smsQuery->where('created_at', '<=', $format_date_to);
                }
            }

            if($search_customer_group){
                $customerId = CustomerGroupCustomer::whereIn('customer_group_id', $search_customer_group)->pluck('customer_id')->toArray();
               
                $smsQuery->whereIn('customer_id',$customerId);
            }

            if($search_campaign){
                $smsQuery->whereIn('campaign_id',$search_campaign);
            }

            if ($search_msg_type != "") {
                $smsQuery->where('type', $search_msg_type);
            }
            
            if ($search_msg_status != "") {
                $smsQuery->where('status', $search_msg_status);
            }
            
            if ($search_msg_delivery_status != "") {
                $smsQuery->where('deliveryStatus', $search_msg_delivery_status);
            }

            $smsQuery->where('broadcast_id','!=',null);

            $smsHistory = $smsQuery->orderBy('created_at','Desc')->with('campaign','customer','broadcast')->withCount('customerGroupCustomerBroadcast')->get();
           
        }else{
            $smsHistory = WhatsappSmsHistory::where('broadcast_id','!=',null)->orderBy('created_at','Desc')->with('campaign','customer','broadcast')->withCount('customerGroupCustomerBroadcast')->get();
        }
     
        

        $output= Excel::create('whatsapp-sms-list', function($excel) use ($smsHistory) {
            
            $excel->sheet('sheet1', function($sheet) use ($smsHistory)
            {
                $sheet->setColumnFormat(array(
                    'D' => '0',
                ));
                
                $sheet->mergeCells('A1:H2');
                $sheet->cell('A1', function($cell) {
                    $cell->setValue('GULSHAN CLUB LIMITED');  
                    $cell->setBackground('#33CCCC');  
                    $cell->setAlignment('center');
                    $cell->setFontWeight('bold');    
                    $cell->setFontSize('20');    
                });

                $sheet->mergeCells('A3:H4');
                $sheet->cell('A3', function($cell) use ($smsHistory) {
                    
                    if(count($smsHistory)>0){
                        $count = count($smsHistory);
                        $lastArray = $count-1;
                        $excelFromDate =date('Y/m/d', strtotime($smsHistory[$lastArray]['created_at']));
                        $excelToDate = date('Y/m/d', strtotime($smsHistory[0]['created_at']));
                    }else{
                        $excelFromDate='N/A';
                        $excelToDate = 'N/A';
                    }
                    

                    $cell->setValue('Whatsapp Message List('.$excelFromDate.' to '.$excelToDate.')');  
                    $cell->setBackground('#008686');  
                    $cell->setAlignment('center');
                    $cell->setFontWeight('bold');    
                    $cell->setFontSize('17');    
                });

               
                
                $sheet->cell('A6', function($cell) {$cell->setValue('No.');  $cell->setFontWeight('bold');});
                $sheet->cell('B6', function($cell) {$cell->setValue('Message Type');  $cell->setFontWeight('bold');});
                $sheet->cell('C6', function($cell) {$cell->setValue('Sms created Date'); $cell->setFontWeight('bold');  });
                $sheet->cell('D6', function($cell) {$cell->setValue('To'); $cell->setFontWeight('bold');  });
                $sheet->cell('E6', function($cell) {$cell->setValue('Number'); $cell->setFontWeight('bold');  });
                $sheet->cell('F6', function($cell) {$cell->setValue('Message'); $cell->setFontWeight('bold');  });
                $sheet->cell('G6', function($cell) {$cell->setValue('Sms Status'); $cell->setFontWeight('bold');  });
                $sheet->cell('H6', function($cell) {$cell->setValue('Delivery Status'); $cell->setFontWeight('bold');  });

                $i=7;
                
                foreach ($smsHistory as $key => $value) {
                    $sheet->cell('A'.$i, $key+1); 
                    $sheet->cell('B'.$i, $value['type']); 
                    $sheet->cell('C'.$i, date('Y-m-d H:i A', (strtotime($value['created_at'])))); 
                    $sheet->cell('D'.$i, $value['broadcast']->broadcast_name);
                    $sheet->cell('E'.$i, $value['customer_group_customer_broadcast_count']); 
                    $sheet->cell('F'.$i, $value['message']); 
                    $sheet->cell('G'.$i, $value['status']); 
                    $sheet->cell('H'.$i, $value['deliveryStatus']); 
                    $i++;
                }

                $i++;
                $sheet->mergeCells('A'.$i.':H'.$i);
                $sheet->cell('A'.$i, function($cell) {
                    $cell->setValue('This file is software generated');  
                    $cell->setBackground('#6666CC');  
                    $cell->setAlignment('center');
                });
                $i++;
                $sheet->mergeCells('A'.$i.':H'.$i);
                $sheet->cell('A'.$i, function($cell) {
                    $cell->setValue('Powered By WORKSPACE INFOTECH LIMITED');  
                    $cell->setBackground('#33CCCC');  
                    $cell->setAlignment('center');
                });
            });
       })->download('xlsx');
    }


    public function whatsappBulkSmsHistory(Request $request)
    {
        $limit = 10;
        $search_phone_number = '';
        $search_customer_group = [];
        $search_campaign = [];
        $search_msg_type = "";
        $search_date_from = "";
        $search_date_to = "";
        $search_msg_status = "";
        $search_msg_delivery_status = "";

        $customerGroup = CustomerGroup::orderBy('created_at', 'desc')->get();
        $campaignList = WhatsappCampaignHistory::orderBy('created_at', 'desc')->get();
        
        
        if($request->phone_number || $request->customer_group || $request->campaigns || $request->date_from 
        ||$request->date_to || $request->msg_delivery_status || $request->msg_status){
            
            $search_phone_number = $request->phone_number;
            $search_user_name = $request->user_name;
            $search_code = $request->code;
            if($request->customer_group){
                $search_customer_group = $request->customer_group;
            }
            
            if($request->campaigns){
                $search_campaign = $request->campaigns;
            }

            if($request->msg_type){
                $search_msg_type = $request->msg_type;
            }
            if($request->date_from){
                $search_date_from = $request->date_from;
            }
            
            if($request->date_to){
                $search_date_to = $request->date_to;
            }
            
            if($request->msg_status){
                $search_msg_status = $request->msg_status;
            }
            
            if($request->msg_delivery_status){
                $search_msg_delivery_status = $request->msg_delivery_status;
            }
            
            
            $smsQuery = WhatsappSmsHistory::query();
            $smsQuery->where('broadcast_id','!=',null);
            if ($request->phone_number != '') {
                $smsQuery->where('phone','!=','');
            }

            if($request->date_from || $request->date_to){
                $format_date_from = date('Y-m-d H:i:s', strtotime($search_date_from));
                $format_date_to = date('Y-m-d 23:59:59', strtotime($search_date_to));

                if ($search_date_from != '' && $search_date_to != '') {
                    $smsQuery->whereBetween('created_at', [$format_date_from, $format_date_to]);
                }
                if ($search_date_from != '' && $search_date_to == '') {
                    $smsQuery->where('created_at', '>=', $format_date_from);
                }
                if ($search_date_from == '' && $search_date_to != '') {
                    $smsQuery->where('created_at', '<=', $format_date_to);
                }
            }

            if($search_customer_group){
                // $customerId = CustomerGroupCustomer::whereIn('customer_group_id', $search_customer_group)->pluck('customer_id')->toArray();
               
                $smsQuery->whereIn('group_id',$search_customer_group);
            }
            
            if($search_campaign){
                $smsQuery->whereIn('campaign_id',$search_campaign);
            }

            if ($search_msg_type != "") {
                $smsQuery->where('type', $search_msg_type);
            }
            
            if ($search_msg_status != "") {
                $smsQuery->where('status', $search_msg_status);
            }
            
            if ($search_msg_delivery_status != "") {
                $smsQuery->where('deliveryStatus', $search_msg_delivery_status);
            }

            $smsHistory = $smsQuery->orderBy('created_at','Desc')->with('campaign','customer','group','broadcast')->withCount('customerGroupCustomerBroadcast')->paginate($limit);
           
        }else{
            $smsHistory = WhatsappSmsHistory::where('broadcast_id','!=',null)->orderBy('created_at','Desc')->with('campaign','customer','group','broadcast')->withCount('customerGroupCustomerBroadcast')->paginate($limit);
        }
        
        // return $smsHistory;

        $data["smsHistory"] = $smsHistory;
        $data["customerGroup"] = $customerGroup;
        $data["campaignList"] = $campaignList;
        $data["page"] = $smsHistory->currentPage();
        $data["limit"] = $limit;
        $data["type"] = $search_msg_type;
        $data["date_from"] = $search_date_from;
        $data["date_to"] = $search_date_to;
        $data["status"] = $search_msg_status;
        $data["deliveryStatus"] = $search_msg_delivery_status;
        $data["search_customer_group"] = $search_customer_group;
        $data["search_campaign"] = $search_campaign;
        $data["search_phone_number"] = $search_phone_number;
        $data["search_customer_group_string"] = $search_customer_group && count($search_customer_group)>0 ? implode(",",$search_customer_group):'';
        $data["search_campaign_string"] = $search_campaign && count($search_campaign)>0 ? implode(",",$search_campaign):'';

        return view('whatsapp.whatsapp-bulk-sms-history', $data);
    }

    public function status_update_whatsapp_sms(Request $request){
        try {
            $success =0;
            $id = $request->id;
            $status = $request->status;
            $res = WhatsappSmsHistory::where('id', $id)->update(['status'=> $status]);
            if($res){
                $success = 1;
            }
            return response(['success' => $success , 'success_status'=>$status, 'pass_id'=>$id]);
        }catch (\Throwable  $e) {
            return response(['success'=>$success, 'error_message'=>$e->getMessage()]);
        }
    }

    public function seen_notification(Request $request){
        try {
            $success =0;
            $id = $request->id;
            $user_id = $request->user_id;
            $notify = Notification::where([['id', $id],['user_id', $user_id],['is_read', 0]])->update(['is_read' => 1]);
            if($notify){
                $success = 1;
            }
            return response(['success' => $success]);
        }catch (\Throwable  $e) {
            return response(['success'=>$success, 'error_message'=>$e->getMessage()]);
        }
    }

    public function delivery_status_update_whatsapp_sms(Request $request){
        try {
            $success =0;
            $id = $request->id;
            $deliveryStatus = $request->delivery_status;
            $res = WhatsappSmsHistory::where('id', $id)->update(['deliveryStatus'=> $deliveryStatus]);
            if($res){
                $success = 1;
            }
            return response(['success' => $success , 'success_status'=>$deliveryStatus, 'pass_id'=>$id]);
        }catch (\Throwable  $e) {
            return response(['success'=>$success, 'error_message'=>$e->getMessage()]);
        }
    }

    public function downloadMediaFile(Request $request){
        try {
            $media_url = $request->media_url;
            $pathToFile = public_path(). "/assets/".$media_url;
            return Response::download($pathToFile);
        }catch (\Throwable  $e) {
            return redirect()->back()->with('alert-danger', $e->getMessage());
        }
    }

    public function resendBulkSms(Request $request){
        $id = $request->id;
        // $id = 4;
        $sms = WhatsappSmsHistory::find($id);
        $sms->cron_processing_status = false;
        $sms->status = "queued";
        $sms->deliveryStatus = "queued";
        $sms->save();
        $this->webhookController->manageUnsentMessage();

        return "success";
    }

    public function whatsappCampaignSmsHistory(Request $request)
    {
        $limit = 10;

        $customerGroup = CustomerGroup::orderBy('created_at', 'desc')->get();
        $campaignList = WhatsappCampaignHistory::orderBy('created_at', 'desc')->with('whatsapp_sms')->paginate($limit);

        $data["customerGroup"] = $customerGroup;
        $data["campaignList"] = $campaignList;
        $data["page"] = $campaignList->currentPage();
        $data["limit"] = $limit;

        return view('whatsapp.whatsapp-campaign-sms-history', $data);
    }


    
}
